/*
 * The codes are used to generate random variables
 * for given distributions.
 */
package PLBP;

/**
 *
 * @author Chen Jiongze
 */
public class RVGenerator {
     /**
     * @param mean mean of the Exponential random variable.
     * @return Exponential random variable.
     */
    public static double getExpRV(double mean){
        double rnd = Math.random();
        while(rnd==0.0){
            rnd = Math.random();
        }
        return -mean*Math.log(rnd);       
    }
     
    /**
     * The method has been given by Knuth.
     * @param mean mean of Poisson Random Variable, usually notated by lambda.
     * @return Poisson random variable.
     */
    public static int getPoissonRV(double mean){
        double L = Math.exp(-mean);
        double p = 1.0;
        int k = 0;
        do{
            k++;
            double rnd = Math.random();
            while(rnd==0.0){
                rnd = Math.random();
            }
            p *= rnd;
        }while(p > L);
        return k - 1;
    }
    
    /**
     * @param gamma shape parameter.
     * @param delta scale parameter.
     * @return Pareto random variable.
     */
    public static double getParetoRV(double gamma, double delta){
        double rnd = Math.random();
        while(rnd==0.0){
            rnd = Math.random();
        }
        return delta * Math.pow(rnd, -1.0/gamma); 
    }
    
    /**
     * @param gamma shape parameter.
     * @param delta scale parameter.
     * @return Forward Recurrence Time Pareto random variable.
     */
    public static double getFRTParetoRV(double gamma, double delta){
        double rnd = Math.random();
        while(rnd==0.0){
            rnd = Math.random();
        }
        if(rnd <= 1.0/gamma){
            return delta * Math.pow(gamma*rnd, 1.0/(1.0-gamma));
        }else{
            return delta * gamma * (1.0-rnd) / (gamma-1.0);
        }
    }
    
    /**
     * @param gamma shape parameter.
     * @param delta scale parameter.
     * @return Lomax random variable.
     */
    public static double getLomaxRV(double gamma, double delta){
        double rnd = Math.random();
        while(rnd==0.0){
            rnd = Math.random();
        }
        return delta * (Math.pow(rnd, -1.0/gamma) - 1.0); 
    }
    
    /**
     * @param gamma shape parameter.
     * @param delta scale parameter.
     * @return Forward Recurrence Time Lomax random variable.
     */
    public static double getFRTLomaxRV(double gamma, double delta){
        double rnd = Math.random();
        while(rnd==0.0){
            rnd = Math.random();
        }
        return delta * (Math.pow(rnd, 1.0/(1.0-gamma))-1); 
    }
}
