//header file.

#ifndef QSFORM_H
#define QSFORM_H

#include <iostream>
#include <math.h>

using namespace std;

const double pi = 3.141592653589793E+00;

const int True = 0;
const int False = 1;

const int ITMAX = 10000;
const double EPS = 3.0e-7;
const double FPMIN = 1.0e-60;   // Number near the smallest
                                // floating-point value.
const double MinAccuracy = 1e-6;
const double GOLD = 1.618034;
const double R = 0.61803399;
const double GOLDC = (1.0 - R);
const double GLIMIT = 100.0;
const double TINY = 1.0e-20;

static double maxarg1,maxarg2;
#define DMAX(a, b) (maxarg1 = (a), maxarg2 = (b), (maxarg1) > (maxarg2) ? (maxarg1) : (maxarg2))

#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))
#define SHFT(a, b, c, d) (a) = (b); (b) = (c); (c) = (d);
#define SHFT2(a, b, c) (a) = (b); (b) = (c);


class QuasiFormula
{
private:
  double lambda;
  double r;
  double delta;
  double Gamma;
  double kappa;
  double Lomaxmean;	//change: Paretomean to Lomaxmean
  double noCBRmean;
  double mean;
  double variance;

  void MaxQSBracket(double *aW, double *bW, double *cW,
        double *lossa, double *lossb, double *lossc,
        double x, double servrate);

  double GoldenQS(double aW, double bW, double cW,
        double x, double servrate, double *Wmax);


public:
  QuasiFormula();
  QuasiFormula(double l, double rVal, double d, double g, double k);
  ~QuasiFormula();

  double GetMean() const;
  double GetNoCBRMean() const;
  double Calcvt(double t);
  double CalcModvt(double t, double W);
  double Calcvprime(double t);
  double CalcModvprime(double t, double W);
  double FindtStar(double x, double W);
  double CalcAMNEstimate(double t, double x, double m, double c, double W);
  double CalcQSForW(double W, double x, double servrate);
  double CalcQSEstimate(double x, double servrate, double *maxW);
  
  ostream& Show(ostream& out);
};

extern double CalculatePoisson(double mean, int n);
extern double CalcPoissonComplement(double mean, int n);
extern void gser(double *gamser, double a, double x, double *gln);
extern void gcf(double *gammcf, double a, double x, double *gln);
extern double gammln(double xx);
extern double Phi(double x, double sigma);
extern double Phibar(double x, double sigma);

extern ostream& operator<<(ostream& out, QuasiFormula& qsf);
#endif
