
#include "qsform.h"
#include <fstream>

/* Use the quai-stationary formula as developed in the paper
 * (Performance Evaluation of a Queue Fed by a Poisson Lomax Burst Process.)  
 * Results are produced for arrange of buffer sizes for each of a range of service rates. 
 * Key parameters are set using constant declarations at the start
 * of this file. Calculations are mostly performed in qsform.c
 *
 */


/*
 * Sample set of parameters
 */
double r = 0.0597;
double lambda = 100.0;
double delta = 1.537;
double Gamma = 1.5;
double overallMean = 0.0;
double minServRate = 3.0;
double maxServRate = 3.0;
double ServRateStep = 3.0;
double maxBuffSize = 30;
double BuffSizeStep = 0.1;


/*
 * This is an example of generating a file containing the overflow
 * probabilities for a given set of parameters
 */
int main(int argc, char *argv[])
{
  double tStar;
  double LomaxMean = delta/(Gamma - 1.0);
  double mean = r*lambda*LomaxMean;
  double variance;
  double kappa = overallMean - mean;
  double maxW, val;
  double xPow;

  QuasiFormula *qf;
  ofstream myfile;
  //output file
  string filename = "PLBPTim-lam100-gamma1_5-delta1_537-r0_0597-net3-t0_1.txt";
  myfile.open (filename);

  qf = new QuasiFormula(lambda, r, delta, Gamma, kappa);

  double tmp = qf->Calcvt(0.1);
  cout << tmp << endl;

  for(double servrate = minServRate; servrate <= maxServRate; 
		servrate += ServRateStep)
    {
    cout << *qf << endl;
    cout << "Service rate \t" << servrate << endl << endl;

     /*The formula cannot be calculated for a zero buffer size,
     so the buffer sizes start at a small, but non-zero, value.*/
    for(double x = 0.1; x < maxBuffSize + BuffSizeStep; x += BuffSizeStep)
      {
      val = qf->CalcQSEstimate(x, servrate, &maxW);
      cout << x << "\t" << val << "\t" << maxW << endl;
	  myfile << x << "\t" << val << endl;
      }
   }
  myfile.close();
}

